@@ -1,6 +1,4 @@
 /*
-        g_object_unref(viewer->priv->src_pixbuf);
-        viewer->priv->src_pixbuf = NULL;
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -21,6 +19,7 @@
 #include <gtk/gtkmarshal.h>
 #include <string.h>
 #include <thunar-vfs/thunar-vfs.h>
+#include <libexif/exif-data.h>
 
 #include "navigator.h"
 #include "picture_viewer.h"
@@ -34,6 +33,7 @@ struct _RsttoPictureViewerPriv
     gboolean          show_border;
     GTimer           *timer;
     gint              timeout_id;
+    gboolean          timeout;
 };
 
 static void
@@ -64,7 +64,7 @@ static gboolean
 rstto_picture_viewer_set_scroll_adjustments(RsttoPictureViewer *, GtkAdjustment *, GtkAdjustment *);
 
 static void
-cb_rstto_picture_viewer_nav_file_changed(RsttoNavigator *, gint , RsttoNavigatorEntry *, RsttoPictureViewer *);
+cb_rstto_picture_viewer_nav_iter_changed(RsttoNavigator *, gint , RsttoNavigatorEntry *, RsttoPictureViewer *);
 static void
 cb_rstto_picture_viewer_value_changed(GtkAdjustment *, RsttoPictureViewer *);
 static void
@@ -113,6 +113,7 @@ rstto_picture_viewer_init(RsttoPictureViewer *viewer)
 
     viewer->priv->show_border = TRUE;
     viewer->priv->timer = g_timer_new();
+    viewer->priv->timeout = TRUE;
 
     g_signal_connect(G_OBJECT(viewer), "scroll_event", G_CALLBACK(cb_rstto_picture_viewer_scroll_event), NULL);
 }
@@ -322,9 +323,7 @@ rstto_picture_viewer_paint(GtkWidget *widget)
                         widget->allocation.height);
         g_object_unref(buffer);
 
-        cursor = gdk_cursor_new(GDK_LEFT_PTR);
-        gdk_window_set_cursor(widget->window, cursor);
-        gdk_cursor_unref(cursor);
+        gdk_window_set_cursor(widget->window, NULL);
     }
 }
 
@@ -410,10 +409,25 @@ cb_rstto_picture_viewer_value_changed(GtkAdjustment *adjustment, RsttoPictureVie
     {
         gint dst_width = gdk_pixbuf_get_width(tmp_pixbuf)*scale;
         gint dst_height = gdk_pixbuf_get_height(tmp_pixbuf)*scale;
-        viewer->priv->dst_pixbuf = gdk_pixbuf_scale_simple(tmp_pixbuf,
+        if (scale < 1.0)
+        {
+            viewer->priv->dst_pixbuf = gdk_pixbuf_scale_simple(tmp_pixbuf,
                                 dst_width>0?dst_width:1,
                                 dst_height>0?dst_height:1,
                                 GDK_INTERP_BILINEAR);
+        }
+        if (scale > 1.0)
+        {
+            viewer->priv->dst_pixbuf = gdk_pixbuf_scale_simple(tmp_pixbuf,
+                                dst_width>0?dst_width:1,
+                                dst_height>0?dst_height:1,
+                                GDK_INTERP_NEAREST);
+        }
+        if (scale == 1.0)
+        {
+            viewer->priv->dst_pixbuf = tmp_pixbuf;
+            g_object_ref(viewer->priv->dst_pixbuf);
+        }
         g_object_unref(tmp_pixbuf);
         tmp_pixbuf = NULL;
     }
@@ -427,7 +441,7 @@ rstto_picture_viewer_new(RsttoNavigator *navigator)
 
     widget = g_object_new(RSTTO_TYPE_PICTURE_VIEWER, NULL);
     RSTTO_PICTURE_VIEWER(widget)->priv->navigator = navigator;
-    g_signal_connect(G_OBJECT(navigator), "iter-changed", G_CALLBACK(cb_rstto_picture_viewer_nav_file_changed), widget);
+    g_signal_connect(G_OBJECT(navigator), "iter-changed", G_CALLBACK(cb_rstto_picture_viewer_nav_iter_changed), widget);
 
     return widget;
 }
@@ -621,9 +635,7 @@ rstto_picture_viewer_refresh(RsttoPictureViewer *viewer)
             if (vadjustment_changed == TRUE)
                 gtk_adjustment_value_changed(viewer->vadjustment);
                 
-            cursor = gdk_cursor_new(GDK_LEFT_PTR);
-            gdk_window_set_cursor(widget->window, cursor);
-            gdk_cursor_unref(cursor);
+            gdk_window_set_cursor(widget->window, NULL);
         }
     }
     else
@@ -640,10 +652,10 @@ rstto_picture_viewer_refresh(RsttoPictureViewer *viewer)
 static gboolean
 cb_rstto_picture_viewer_update_image(RsttoPictureViewer *viewer)
 {
-    gulong millisec = 0;
+    gdouble sec;
 
-    g_timer_elapsed(viewer->priv->timer, &millisec);
-    if (millisec >= 300)
+    sec = g_timer_elapsed(viewer->priv->timer, NULL);
+    if (sec >= 0.250)
     {
         g_timer_stop(viewer->priv->timer);
         viewer->priv->timeout_id = 0;
@@ -654,7 +666,7 @@ cb_rstto_picture_viewer_update_image(RsttoPictureViewer *viewer)
 }
 
 static void
-cb_rstto_picture_viewer_nav_file_changed(RsttoNavigator *nav, gint nr, RsttoNavigatorEntry *entry, RsttoPictureViewer *viewer)
+cb_rstto_picture_viewer_nav_iter_changed(RsttoNavigator *nav, gint nr, RsttoNavigatorEntry *entry, RsttoPictureViewer *viewer)
 {
     GtkWidget *widget = GTK_WIDGET(viewer);
     if(entry)
@@ -665,16 +677,46 @@ cb_rstto_picture_viewer_nav_file_changed(RsttoNavigator *nav, gint nr, RsttoNavi
             gdk_window_set_cursor(widget->window, cursor);
             gdk_cursor_unref(cursor);
         }
-        g_timer_start(viewer->priv->timer);
-        if (viewer->priv->timeout_id == 0)
-            viewer->priv->timeout_id = g_timeout_add(300, (GSourceFunc)cb_rstto_picture_viewer_update_image, viewer);
+        if (viewer->priv->timeout == TRUE)
+        {
+            if (viewer->priv->timeout_id == 0)
+            {
+                if (viewer->priv->dst_pixbuf)
+                {
+                    GdkPixbuf *pixbuf = gdk_pixbuf_composite_color_simple (viewer->priv->dst_pixbuf,
+                                                                    gdk_pixbuf_get_width(viewer->priv->dst_pixbuf),
+                                                                    gdk_pixbuf_get_height(viewer->priv->dst_pixbuf),
+                                                                    GDK_INTERP_BILINEAR,
+                                                                    100,
+                                                                    100,
+                                                                    0x000000,
+                                                                    0x000000);
+                    gdk_pixbuf_unref(viewer->priv->dst_pixbuf);
+                    viewer->priv->dst_pixbuf = pixbuf;
+                                                                    
+                    rstto_picture_viewer_paint(GTK_WIDGET(viewer));
+                }
+                viewer->priv->timeout_id = g_timeout_add(100, (GSourceFunc)cb_rstto_picture_viewer_update_image, viewer);
+            }
+            g_timer_start(viewer->priv->timer);
+        }
+        else
+        {
+            rstto_picture_viewer_update(viewer);
+        }
     }
     else
     {
-        g_object_unref(viewer->priv->src_pixbuf);
-        viewer->priv->src_pixbuf = NULL;
-        g_object_unref(viewer->priv->dst_pixbuf);
-        viewer->priv->dst_pixbuf = NULL;
+        if (viewer->priv->src_pixbuf)
+        {
+            g_object_unref(viewer->priv->src_pixbuf);
+            viewer->priv->src_pixbuf = NULL;
+        }
+        if (viewer->priv->dst_pixbuf)
+        {
+            g_object_unref(viewer->priv->dst_pixbuf);
+            viewer->priv->dst_pixbuf = NULL;
+        }
         if (GTK_WIDGET_REALIZED(widget))
         {
             rstto_picture_viewer_paint(GTK_WIDGET(viewer));
@@ -703,7 +745,8 @@ rstto_picture_viewer_update(RsttoPictureViewer *viewer)
     if(entry)
     {
         viewer->priv->src_pixbuf = rstto_navigator_entry_get_pixbuf(entry);
-        g_object_ref(viewer->priv->src_pixbuf);
+        if (viewer->priv->src_pixbuf)
+            g_object_ref(viewer->priv->src_pixbuf);
     }
 
     if(GTK_WIDGET_REALIZED(widget))
@@ -711,9 +754,7 @@ rstto_picture_viewer_update(RsttoPictureViewer *viewer)
         rstto_picture_viewer_refresh(viewer);
         rstto_picture_viewer_paint(GTK_WIDGET(viewer));
 
-        GdkCursor *cursor = gdk_cursor_new(GDK_LEFT_PTR);
-        gdk_window_set_cursor(widget->window, cursor);
-        gdk_cursor_unref(cursor);
+        gdk_window_set_cursor(widget->window, NULL);
     }
 }
 
@@ -748,3 +789,9 @@ cb_rstto_picture_viewer_scroll_event (RsttoPictureViewer *viewer, GdkEventScroll
         rstto_picture_viewer_paint(GTK_WIDGET(viewer));
     }
 }
+
+void
+rstto_picture_viewer_set_timeout(RsttoPictureViewer *viewer, gboolean timeout)
+{
+    viewer->priv->timeout = timeout;
+}
